"""
Helpers for generating SQL queries

These helpers are here to aid in generating simple and common SQL
queries. For more interresting queries you have to build the query
yourself. 
"""
import WebFrame as WF

class ParameterProblem(Exception):pass

def params_to_string(params, sep=',',nullmatch="="):
    """Builds a 'sep'-separated param=value list from the dict in 'params'.
    Can be used to build the list after SET, or a condition list for WHERE.
    'nullmatch' is the string to be used when the value is null, since
    SQL doesn't really do what you want if you have WHERE a = NULL. So
    you need to change it to WHERE a IS NULL.
    """
    dbh = WF.open_db()

    set = None
    for k in params:
        if set: set += " "+sep+" "
        else: set = ""
        set += dbh.quote_table(k)
        if params[k] == None:
            set += " "+nullmatch+" "
        else:
            set += "="
        set += dbh.quote(params[k])

    return set

def make_update(table, params, keys=None, allrows=False):
    """Make an UPDATE query that changes the params in "params".
    keys are the params that are put after the WHERE clause.
    Keys are required unless allrows=True. Raises ParameterProblem.
    """
    dbh = WF.open_db()

    # Check key parameter
    if allrows == False and (keys == None or len(keys) == 0):
        raise ParameterProblem("Keys not specified")

    q = "UPDATE "+dbh.quote_table(table)+" SET "
    q+= params_to_string(params, ",")
    if not allrows:
        q+= " WHERE "
        q+= params_to_string(keys, "AND", "IS")

    return q


def make_replace(table, params):
    """Make a REPLACE query that changes the params in "params".
    """
    dbh = WF.open_db()

    q = "REPLACE INTO "+dbh.quote_table(table)+" SET "
    q+= params_to_string(params, ",")

    return q


def make_insert(table, params, duplicate_key_update=False, ignore=False):
    """Make an INSERT query from 'params'.
    If 'duplicate_key_update' is True then 'ON DUPLICATE KEY
    UPDATE' is added to the query.
    If 'ignore' is True the query becomes an INSERT IGNORE.
    """
    dbh = WF.open_db()

    set = params_to_string(params, ",")

    if ignore:
        q = "INSERT IGNORE INTO"
    else:
        q = "INSERT INTO "
    q += dbh.quote_table(table)+" SET " + set

    if duplicate_key_update:
        q += " ON DUPLICATE KEY UPDATE " + set

    return q


def make_delete(table, keys=None, allrows=False):
    """Make a DELETE query.
    keys are the params that are put after the WHERE clause.
    Keys are required unless allrows=True. Raises ParameterProblem.
    """
    dbh = WF.open_db()

    # Check key parameter
    if allrows == False and (keys == None or len(keys) == 0):
        raise ParameterProblem("Keys not specified")

    q = "DELETE FROM "
    q += dbh.quote_table(table)
    if not allrows:
        q+= " WHERE "
        q+= params_to_string(keys, "AND", "IS")

    return q
