"""
The Database wrapper module.
This module provides a wrapper for a database class adding some functions.
The prefered use is as folows:
Open master db:
    import WebFrame as WF
    dbh = WF.open_db()

For SELECT:
    cr = dbh.simple("SELECT some, thing FROM Whatever")
    for res in cr:
        # Iterate over results
        do_something(res["some"])
        do_something(res["thing"])

For INSERT/UPDATE/DELETE/REPLACE/etc:
    try:
        dbh.simple("INSERT INTO Whatever VALUES(23)")
        dbh.simple("INSERT INTO Whatever VALUES(42)")
    except dbh.Error, e:
        dbh.rollback()
        print "FUBAR", e
    else:
        dbh.commit()
"""
import MySQLdb
import MySQLdb.connections
import MySQLdb.cursors
import WebFrame as WF
import logging

class Cursor(MySQLdb.cursors.Cursor):
    """Our improved mysql cursor class
    this class returns dicts when iterated over"""
    def _dictify(self, row):
        """Convert 'row' from 'cursor' into a dictionary with
        the column names as keys"""
        if row == None: return None
        seq = map(lambda x,y: (x[0], y),
                  self.description,
                  row)
        return dict(seq)

    def __iter__(self):
        row = self.fetchone()
        while row != None:
            yield row
            row = self.fetchone()

    def fetchone(self):
        """Fetch one row from the result set.
        The row is translated into a dict with the column names
        as keys"""
        row = MySQLdb.cursors.Cursor.fetchone(self)
        return self._dictify(row)


class DB(MySQLdb.connections.Connection):
    def __init__(self, host, name, user, password):
        """
        Open a database connection.
        You shouldn't actually create these classes yourself
        but instead use WebFrame.open_db() to open the main db.
        """
        MySQLdb.connections.Connection.__init__(self, db=name, host=host,
                                                user=user, passwd=password,
                                                cursorclass=Cursor,
                                                use_unicode=True,
                                                charset="utf8")
        self.autocommit(False)
                                

    def simple(self, query, *params):
        """A simple SQL query.
        Works great for SELECT and other read-queries,
        but for write-queries (INSERT, UPDATE etc) you need to call
        res.commit() to actually commit the data. (or res.rollback()
        if you changed your mind"""

        if len(params) == 1:
            params = params[0]
            
        if WF.get_opt("debug"):
            logging.debug("SQL: [%s]", query % self.literal(params))
        cr = self.cursor()
        cr.execute(query, params)
        return cr


    def quote(self, param):
        """Quotes a parameter for SQL"""
        return self.literal(param)

    def quote_table(self, tbl):
        """Quotes a table spec.
        For example Data.Value gets quoted to `Data`.`Value`
        """
        parts = tbl.split(".")
        parts = ["`"+str(p)+"`" for p in parts]

        return ".".join(parts)
