import time
import logging
import WebFrame as WF
from WebFrame import handler
from WebFrame import template
import demoapp.cache
import demoapp.helper
try:
    from xml.etree import cElementTree as ET
except ImportError:
    try:
        from elementtree import cElementTree as ET
    except ImportError:
        from elementtree import ElementTree as ET


# Set config options for this app.
# These are listed in WebFrame.options and documented using pydoc
WF.set_opt("file.default_encoding", "iso-8859-1")
WF.set_opt("http.default_encoding", "iso-8859-1")
WF.set_opt("template.allow_pathinfo", True)

###############################################################################
### Example of path-based handlers.
### These are triggered for paths matching a prefix, and a regular expression
### So a handler for path /a/path and regexp [0-9]+ would trigger for all
### URI's of the form /a/path/<number>
###
### See WebFrame.handler for details
###############################################################################
####
# A "plain" handler. The system doesn't do any processing of the request body
# other than parsing out form data if it is a form submit.
# This handlers read-access policy is "@public" and
# write-acess policy is "@deny"
def h_plain(ctx):
    ctx.resp.write("PathInfo: %s\n" % str(ctx.req.path_info))
    ctx.resp.write("Query: %s\n" % str(ctx.req.params))
    return ctx.resp.stop_processing()
handler.register_plain("ANY", "/manual/plain", ".*", h_plain,
                       "@public", "@deny")
####

####
# A 'JSON' format handler. The data from the request body is parsed as JSON
# by the system and sent to the handler as a 'python structure' as the
# second parameter. Basically the structure is a dict of dicts of lists..etc
# The handler is expected to return a similar 'structure' that is then
# serialized as JSON by the system and sent to the client.
def h_json(ctx, js):
    logging.info("I gots me some json: %s", str(js))
    return { "my" : {"a":u"jj"}, "other":js }
handler.register_json("ANY", "/manual/json", ".*", h_json,
                      "@public", "@public")
####

####
# A 'xml' format handler. As with the JSON handler the system does the
# parsing and serialization of xml.
# The handler receives an elementtree root element as its second argument.
# The handler is expected to return two things:
#   * a content-type that will be sent to the client.
#         (this may change in future versions)
#   * an elementtree root element, that will be serialized into xml and sent
#         to the client.
def h_xml(ctx, etree):
    logging.info("I gots me some xml: %s", str(etree))
    x2 =  ET.Element("yo")
    x2.append(ET.Element("eep"))
    if etree:
        x2.append(etree)
    return "application/x-demo+xml", x2
handler.register_xml("ANY", "/manual/xml", ".*", h_xml,
                     "@public", "@public")
####

####
# This is just a simple 'plain' handler.
# We use it here to demonstrate how to access and set cookies.
def h_cookie(ctx):
    ctx.resp.set_cookie("cookie", "smells great", time.time()+600)
    ctx.resp.write("Got cookies: %s\n" % str(ctx.req.cookies))
    return ctx.resp.stop_processing()
handler.register_plain("ANY", "/manual/cookie", ".*", h_cookie,
                       "@public", "@deny")
####

####
# This is a 'plain' handler that accesses the session object
def h_session(ctx):
    ctx.resp.write("This time session was: %s\n" % str(ctx.sess))
    if "poke" in ctx.sess:
        ctx.sess["poke"] += 1
    else:
        ctx.sess["poke"] = 1
    ctx.resp.write("Now it is: %s\n" % str(ctx.sess))
    return ctx.resp.stop_processing()
handler.register_plain("ANY", "/manual/session", ".*", h_session,
                       "@public", "@deny")
###############################################################################
#### End of handler demos
###############################################################################

###############################################################################
#### Below are some examples of how to setup and access a database.
#### You also need to have the database parameters set if you use the
#### user-login helpers.
####
#### SECURITY: Even tho handlers have access policies that are checked by
####           the system you need to be very careful when writing database
####           access handlers and template-functions!
####           The system only checks a basic policy. It is up to you to make
####           sure users aren't allowed to acces other users data!
####
#### WARNING: Don't use the code below for anything! It is just an example
####          and is not secure!
###############################################################################
# Database setup
WF.set_opt("db.host", "localhost")
WF.set_opt("db.user", "test")
WF.set_opt("db.name", "test")
WF.set_opt("db.pass", "test")

# A sample plain handler showing how to access the database
def sql_injection(ctx):
    """BIG WARNING!! THIS IS A SECURITY RISK!!!
    This functon is for demo only!!!!"""
    db = WF.open_db()
    ctx.resp.write("Asking for [%s]\n" % ctx.req.path_info)
    cr = db.simple(ctx.req.path_info)
    for i in cr:
        ctx.resp.write(unicode(i)+"\n")
    ctx.resp.write("All done")
handler.register_plain("GET", "/sql", ".*", sql_injection,"@public","@public")
###############################################################################
#### End of db demo
###############################################################################



###############################################################################
#### Examples of variables and functions that are exported to the templates
#### these are global for the app, and all templates will get them.
#### Functions have access policies, but variables are accessible by all.
####
#### Template functions get the context as their first argument, and then
#### the parameters sent by the template.
####
#### NOTE1: When writing database access functions make sure to do extra
####        security checks so users don't get access to stuff they shouldn't
#### NOTE2: Database access functions should return an iterator instead of a
####        list when returning a resultset. The default cursor object
####        returned by WebFrame.db is provides an iterator and can be
####        returned by calling iter(...) on it if you don't need to process
####        the result set.
####        Hint: http://heather.cs.ucdavis.edu/~matloff/Python/PyIterGen.pdf
###############################################################################
# Register some template helpers
template.register_value("the_var", 42)
template.register_func("funky", lambda ctx, x: "funky "+str(x),
                       "@loggedin", "@loggedin")

# Register a DB handler, that returns an iterator to the template
# iterators are better since they use less resources.
# Fortunately the database result cursor can provide an iterator =)
# WARNING: Make sure to call iter() on the object, as passing it directly
#          to the template can expose stuff the template shouldn't touch!
def DB_list_sessions(ctx):
    dbh = WF.open_db()
    cr = dbh.simple("SELECT * FROM Session")
    return iter(cr)
template.register_func("DB_list_sessions", DB_list_sessions,
                       "@loggedin", "@loggedin")
###############################################################################
#### End of template functions demo
###############################################################################



###############################################################################
#### Query parameter handlers demo.
#### These handlers are useful when you want to run extra code based on
#### something sent from a form, without having a separate page that the
#### form is submitted to.
#### The system triggers on form parameters beginning with 'ACTION:'
#### (settable via the 'http.query_handler_prefix' option.)
#### and if it finds a matching handler then that handler gets to run before
#### the templates or other handlers.
####
#### The handler may select to abort further processing as the example
#### below does, or it can allow further processing by returning
#### 'ctx.resp.cont()' instead.
###############################################################################

def qhandler(ctx, action):
    ctx.resp.write("Qhandler triggered for %s\n" % action)
    ctx.resp.write("Parameters are: %s\n" % ctx.req.params)
    return ctx.resp.stop_processing()
handler.register_query("demo", qhandler, "@public", "@public")

###############################################################################
#### End of query handler demo
###############################################################################
